package expr

import (
	"context"
	"testing"

	"github.com/stretchr/testify/require"
)

type testEnvStruct struct {
	ID   string      `expr:"id"`
	Size int         `expr:"size"`
	Data interface{} `expr:"data"`
}

func (s *testEnvStruct) Equal(other *testEnvStruct) bool {
	return s.ID == other.ID
}

func TestExpr_Example(t *testing.T) {
	ctx := context.Background()

	tests := []struct {
		name       string
		exp        string
		env        map[string]interface{}
		wantErr    bool
		wantResult interface{}
	}{
		{
			name:       "get field by expr tag",
			exp:        "s.id",
			env:        map[string]interface{}{"s": &testEnvStruct{ID: "id1"}},
			wantResult: "id1",
		},
		{
			name:       "get field by field name",
			exp:        "s.ID",
			env:        map[string]interface{}{"s": &testEnvStruct{ID: "id1"}},
			wantResult: "id1",
		},
		{
			name:       "get nested field",
			exp:        "m.s.size",
			env:        map[string]interface{}{"m": map[string]interface{}{"s": &testEnvStruct{Size: 1}}},
			wantResult: 1,
		},
		{
			name:       "check field",
			exp:        "s.data.size < 100",
			env:        map[string]interface{}{"s": &testEnvStruct{Data: &testEnvStruct{Size: 0}}},
			wantResult: true,
		},
		{
			name:       "use method",
			exp:        "s1.Equal(s2)",
			env:        map[string]interface{}{"s1": &testEnvStruct{ID: "id1"}, "s2": &testEnvStruct{ID: "id2"}},
			wantResult: false,
		},
		{
			name:    "field not exists",
			exp:     "s.not_exists",
			env:     map[string]interface{}{"s": &testEnvStruct{}},
			wantErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			result, err := Eval(ctx, tt.exp, tt.env)
			if tt.wantErr {
				require.Error(t, err)
				return
			}

			require.NoError(t, err)
			require.Equal(t, tt.wantResult, result)
		})
	}
}
